@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008-2018 Sergey Poznyakoff
+   Copyright (C) 2008-2024 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,9 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <dico.h>
 #include <stdio.h>
 #include <string.h>
@@ -75,7 +73,7 @@ dico_strategy_dup(const dico_strategy_t strat)
 dico_strategy_t
 dico_strategy_find(const char *name)
 {
-    if (DEFSTRATNAME(name)) 
+    if (DEFSTRATNAME(name))
 	return default_strategy;
     return dico_list_locate(strategy_list, (void*)name);
 }
@@ -99,7 +97,7 @@ dico_strategy_add(const dico_strategy_t strat)
     return 0;
 }
 
-dico_iterator_t 
+dico_iterator_t
 dico_strategy_iterator(void)
 {
     return dico_list_iterator(strategy_list);
@@ -138,5 +136,3 @@ dico_get_default_strategy(void)
 {
     return default_strategy;
 }
-
-    
